@@ -90,7 +90,7 @@ file.
 
 =head1 DESCRIPTION
 
-StreamFinder::Odysee accepts a valid full Odysee video URL on 
+StreamFinder::Odysee accepts a valid full Odysee video webpage URL on 
 odysee.com and returns the actual stream URL, title, and cover art icon 
 for that video.  The purpose is that one needs this URL in order to have 
 the option to stream the video in one's own choice of media player 
@@ -100,27 +100,40 @@ come with that method of play.  The author uses his own custom all-purpose
 media player called "fauxdacious" (his custom hacked version of the 
 open-source "audacious" audio player).  "fauxdacious" incorporates this 
 module to decode and play odysee.com videos.  This is a submodule of the 
-general StreamFinder module.
+general StreamFinder module.  NOTE:  This submodule requires a full 
+odysee URL as there is no searchability by just the "ID"!
 
 Depends:  
 
 L<URI::Escape>, L<HTML::Entities>, L<LWP::UserAgent>, 
-and the separate application program:  youtube-dl.
+and the separate application program:  youtube-dl, or a compatable program 
+such as yt-dlp.
 
 =head1 SUBROUTINES/METHODS
 
 =over 4
 
-=item B<new>(I<ID>|I<url> [, I<-youtube> => yes|no|only ] 
-[, I<-keep> => "type1,type2?..." | [type1,type2?...] ] 
-[, I<-secure> [ => 0|1 ]] [, I<-debug> [ => 0|1|2 ]])
+=item B<new>(I<ID>|I<url> [, I<-youtube> => yes|no|first|last|only ] 
+[, I<-secure> [ => 0|1 ]] 
+[, I<-debug> [ => 0|1|2 ]])
 
-Accepts a odysee.com video URL and creates and returns a new video object, 
+Accepts a full odysee.com video URL and creates and returns a new video object, 
 or I<undef> if the URL is not a valid Odysee video or no streams are found.  
 The URL must be the full URL, ie. https://odysee.com/@channel/video-id.
 NOTE:  I<channel> and I<video-id> usually end with a colon followed by 
 random digits.
 
+The optional I<-youtube> argument can be set to "I<yes>" or "I<last>" - Odysee 
+will also look in the bottom of the video's description for a youtube link and 
+try to fetch Youtube streams (last - after Odysee streams); "I<no>" - only 
+include streams found from the video's odysee.com page; "I<only>" - return 
+results only if there's a youtube link found at the bottom of the description 
+(use only Youtube streams); or "I<first>" - include the Youtube streams 
+(if any) first.  Default is B<"no"> (ignore any youtube link in 
+the description).  NOTE:  Unlike most other modules, youtube will NOT be 
+checked if I<-youtube> is set to I<no> regardless of whether any Odysee streams 
+were found.
+
 The optional I<-secure> argument can be either 0 or 1 (I<false> or I<true>).  If 1 
 then only secure ("https://") streams will be returned.
 
@@ -128,6 +141,14 @@ DEFAULT I<-secure> is 0 (false) - return all streams (http and https).
 
 Additional options:
 
+Certain youtube-dl (L<StreamFinder::Youtube>) configuration options, 
+namely I<-format>, I<-formatonly>, I<-youtube-dl-args>, 
+and I<-youtube-dl-add-args> can be overridden here by specifying 
+I<-youtube-format>, I<-youtube-formatonly>, I<-youtube-dl-args>, 
+and I<-youtube-dl-add-args> arguments respectively.  It is however, 
+recommended to specify these in the Odysee-specific configuration file 
+(see B<CONFIGURATION FILES> below.
+
 I<-log> => "I<logfile>"
 
 Specify path to a log file.  If a valid and writable file is specified, A line will be 
@@ -211,6 +232,11 @@ Returns the video's type ("Odysee").
 
 =head1 CONFIGURATION FILES
 
+The default root location directory for StreamFinder configuration files 
+is "~/.config/StreamFinder".  To use an alternate location directory, 
+specify it in the "I<STREAMFINDER>" environment variable, ie.:  
+B<$ENV{STREAMFINDER} = "/etc/StreamFinder">.
+
 =over 4
 
 =item ~/.config/StreamFinder/Odysee/config
@@ -218,19 +244,25 @@ Returns the video's type ("Odysee").
 Optional text file for specifying various configuration options 
 for a specific site (submodule).  Each option is specified on a 
 separate line in the format below:
+NOTE:  Do not follow the lines with a semicolon, comma, or any other 
+separator.  Non-numeric I<values> should be surrounded with quotes, either 
+single or double.  Blank lines and lines beginning with a "#" sign as 
+their first non-blank character are ignored as comments.
 
 'option' => 'value' [,]
 
 and the options are loaded into a hash used only by the specific 
 (submodule) specified.  Valid options include 
-I<-debug> => [0|1|2], and most of the L<LWP::UserAgent> options.  
-Blank lines and lines starting with a "#" sign are ignored.
+I<-debug> => [0|1|2]> and most of the L<LWP::UserAgent> options.  
+
+Among options valid for Odysee streams are the various youtube-dl 
+(L<StreamFinder::Youtube>) configuration options, 
+namely I<format>, I<formatonly>, I<youtube-dl-args>, and I<youtube-dl-add-args> 
+can be overridden here by specifying I<youtube-format>, I<youtube-formatonly>, 
+I<youtube-dl-args>, and I<youtube-dl-add-args> arguments respectively.  
 
 Options specified here override any specified in I<~/.config/StreamFinder/config>.
 
-Among options valid for Odysee streams is the I<-keep> and 
-I<-youtube> options described in the B<new()> function.
-
 =item ~/.config/StreamFinder/config
 
 Optional text file for specifying various configuration options.  
@@ -240,12 +272,12 @@ Each option is specified on a separate line in the format below:
 
 and the options are loaded into a hash used by all sites 
 (submodules) that support them.  Valid options include 
-I<-debug> => [0|1|2], and most of the L<LWP::UserAgent> options.
+I<-debug> => [0|1|2]> and most of the L<LWP::UserAgent> options.
 
 =back
 
-NOTE:  Options specified in the options parameter list will override 
-those corresponding options specified in these files.
+NOTE:  Options specified in the options parameter list of the I<new()> 
+function will override those corresponding options specified in these files.
 
 =head1 KEYWORDS
 
@@ -255,7 +287,9 @@ odysee
 
 L<URI::Escape>, L<HTML::Entities>, L<LWP::UserAgent>
 
-youtube-dl
+youtube-dl (or yt-dlp, or other compatable program)
+
+=head1 RECCOMENDS
 
 wget
 
@@ -355,6 +389,15 @@ sub new
 
 	my $self = $class->SUPER::new('Odysee', @_);
 	$DEBUG = $self->{'debug'}  if (defined $self->{'debug'});
+	$self->{'youtube'} = 'no'  unless (defined $self->{'youtube'});
+	while (@_) {
+		if ($_[0] =~ /^\-?youtube$/o) {
+			shift;
+			$self->{'youtube'} = (defined $_[0]) ? shift : 'yes';
+		} else {
+			shift;
+		}
+	}
 
 	print STDERR "-0(Odysee): URL=$url=\n"  if ($DEBUG);
 	(my $url2fetch = uri_unescape($url));
@@ -395,46 +438,63 @@ sub new
 		($self->{'imageurl'} = $self->{'iconurl'}) =~ s#^.+?(https?\:\/.+)$#$1#i
 				if ($self->{'iconurl'});
 		$self->{'year'} = $1  if ($jsonstuff =~ m#\"uploadDate\"\:\s*\"(\d\d\d\d)#s);
-		my $streamref = $1  if ($jsonstuff =~ m#\"embedUrl\"\:\s*\"([^\"]+)#s);
-		$streamref ||= $1  if ($jsonstuff =~ m#\"contentUrl\"\:\s*\"([^\"]+)#s);
-		if ($streamref) {
-			print STDERR "\n-2 FETCHING STREAM(S) VIA youtube-dl(REQUIRED)...\n"  if ($DEBUG);
-			my $ytdlArgs = '--get-url --get-thumbnail --get-title --get-description -f "'
-					. ((defined $self->{'format'}) ? $self->{'format'} : 'mp4')
-					. '" ' . ((defined $self->{'youtube-dl-args'}) ? $self->{'youtube-dl-args'} : '');
-			my $try = 1;
-			my ($more, @ytdldata, @ytStreams);
-			my $prevDescription = $self->{'description'};
-			if (defined($self->{'userid'}) && defined($self->{'userpw'})) {  #USER HAS A LOGIN CONFIGURED:
-				my $uid = $self->{'userid'};
-				my $upw = $self->{'userpw'};
-				$_ = `youtube-dl --username "$uid" --password "$upw" $ytdlArgs "$url2fetch"`;
-			} else {
-				$_ = `youtube-dl $ytdlArgs "$url2fetch"`;
+		my $haveYoutube = 0;
+		eval { require 'StreamFinder/Youtube.pm'; $haveYoutube = 1; };
+		print STDERR "\n-2 NO STREAMS FOUND IN PAGE (haveYoutube=$haveYoutube)\n"  if ($DEBUG && $self->{'cnt'} <= 0);
+		if ($haveYoutube) {
+			print STDERR "\n-2 TRYING youtube-dl...\n"  if ($DEBUG);
+			my %globalArgs = (
+					'-noiframes' => 1, '-fast' => 1, '-debug' => $DEBUG
+			);
+			foreach my $arg (qw(secure log logfmt youtube-format youtube-formatonly
+					youtube-dl-args youtube-dl-add-args)) {
+				(my $arg0 = $arg) =~ s/^youtube\-(?!dl)//o;
+				$globalArgs{$arg0} = $self->{$arg}  if (defined $self->{$arg});
 			}
-			print STDERR "--TRY($try of 1): youtube-dl returned=$_= ARGS=$ytdlArgs=\n"  if ($DEBUG);
-			@ytdldata = split /\r?\n/s;
-			return undef unless (scalar(@ytdldata) > 0);
+			my $yt = new StreamFinder::Youtube($url2fetch, %globalArgs);
+			if ($yt) {
+				print STDERR "--PREFER-YT=".$self->{'youtube'}."= iconURL=".$yt->{'iconurl'}."=\n"  if ($DEBUG);
+				if ($self->{'youtube'} =~ /(?:yes|top|first|last|only)/i && $yt->{'iconurl'} =~ /\b(?:youtube\.|youtu.be|ytimg\.)\b/) {
+					#ODYSEE VIDEOS USUALLY HAVE A YOUTUBE URL AS LAST LINE IN DESCRIPTION, WHICH ENDS
+					#UP IN THE iconurl ARGUMENT AND USER WOULD PREFER YOUTUBE, SO WE'LL TRY THAT!:
+					print STDERR "---user prefers Youtube (".$yt->{'iconurl'}.")!...\n"  if ($DEBUG);
+					my %globalArgs = (
+							'-noiframes' => 1, '-debug' => $DEBUG
+					);
+					foreach my $arg (qw(secure log logfmt youtube-format youtube-formatonly
+							youtube-dl-args youtube-dl-add-args)) {
+						(my $arg0 = $arg) =~ s/^youtube\-(?!dl)//o;
+						$globalArgs{$arg0} = $self->{$arg}  if (defined $self->{$arg});
+					}
+					my $yt0 = new StreamFinder::Youtube($yt->{'iconurl'}, %globalArgs);
+					if (defined($yt0) && $yt0->count() > 0) {
+						return $yt0  if ($self->{'youtube'} =~ /only/i);
 
-			#NOTE:  ytdldata is ORDERED:  TITLE?, STREAM-URLS, THEN THE ICON URL, THEN DESCRIPTION!:
-			unless ($ytdldata[0] =~ m#^https?\:\/\/#) {  #SHIFT OFF TITLE FROM TOP (IF NOT A URL!):
-				$_ = shift(@ytdldata);
-				$self->{'title'} ||= $_;
-			}
-			$more = 1;
-			@ytStreams = ();
-			while (@ytdldata) {
-				$_ = shift @ytdldata;
-				$more = 0  unless (m#^https?\:\/\/#o);
-				if ($more) {
-					push @ytStreams, $_  unless ($self->{'secure'} && $_ !~ /^https/o);
-				} else {
-					$self->{'description'} .= $_ . ' '  unless ($prevDescription);
+						my @ytStreams = $yt0->get();
+						push @{$self->{'streams'}}, @ytStreams;
+						if ($self->{'youtube'} =~ /(?:top|first)/i) {
+							foreach my $field (qw(title description artist albumartist year iconurl articonurl)) {
+								$self->{$field} ||= $yt0->{$field}  if (defined($yt0->{$field}) && $yt0->{$field});
+							}
+						}
+						$self->{'cnt'} = scalar @{$self->{'streams'}};
+						print STDERR "i:Found stream(s) (".join('|',@ytStreams).") via youtube-dl.\n"  if ($DEBUG);
+					}					
+				}
+				if ($yt->count() > 0) {
+					my @ytStreams = $yt->get();
+					if ($self->{'youtube'} =~ /(?:top|first)/i) {  #PUT youtube-dl STREAMS ON TOP:
+						push @{$self->{'streams'}}, @ytStreams;
+					} else {
+						unshift @{$self->{'streams'}}, @ytStreams;
+					}
+					foreach my $field (qw(title description)) {
+						$self->{$field} ||= $yt->{$field}  if (defined($yt->{$field}) && $yt->{$field});
+					}
+					$self->{'cnt'} = scalar @{$self->{'streams'}};
+					print STDERR "i:Found stream(s) (".join('|',@ytStreams).") via youtube-dl.\n"  if ($DEBUG);
 				}
 			}
-			my $ytimage = ($#ytStreams > 0) ? pop(@ytStreams) : '';
-			$self->{'iconurl'} ||= $ytimage  if ($ytimage && $ytimage =~ /^http/o);
-			push @{$self->{'streams'}}, @ytStreams;
 			$self->{'imageurl'} ||= $self->{'iconurl'};
 			$self->{'cnt'} = scalar @{$self->{'streams'}};
 			if ($self->{'description'} =~ /\w/) {
